#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies simplest-possible build of a "Hello, world!" program
using the default build target.
"""

import TestGyp

# Android does not support setting the build directory.
test = TestGyp.TestGyp(formats=['!android'])

test.run_gyp('product.gyp')
test.build('product.gyp')

# executables
test.built_file_must_exist('alt1' + test._exe, test.EXECUTABLE, bare=True)
test.built_file_must_exist('hello2.stuff', test.EXECUTABLE, bare=True)
test.built_file_must_exist('yoalt3.stuff', test.EXECUTABLE, bare=True)

# shared libraries
test.built_file_must_exist(test.dll_ + 'alt4' + test._dll,
                           test.SHARED_LIB, bare=True)
test.built_file_must_exist(test.dll_ + 'hello5.stuff',
                           test.SHARED_LIB, bare=True)
test.built_file_must_exist('yoalt6.stuff', test.SHARED_LIB, bare=True)

# static libraries
test.built_file_must_exist(test.lib_ + 'alt7' + test._lib,
                           test.STATIC_LIB, bare=True)
test.built_file_must_exist(test.lib_ + 'hello8.stuff',
                           test.STATIC_LIB, bare=True)
test.built_file_must_exist('yoalt9.stuff', test.STATIC_LIB, bare=True)

# alternate product_dir
test.built_file_must_exist('bob/yoalt10.stuff', test.EXECUTABLE, bare=True)
test.built_file_must_exist('bob/yoalt11.stuff', test.EXECUTABLE, bare=True)
test.built_file_must_exist('bob/yoalt12.stuff', test.EXECUTABLE, bare=True)

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Test cases when multiple targets in different directories have the same name.
"""

import TestGyp

test = TestGyp.TestGyp(formats=['android', 'ninja', 'make'])

test.run_gyp('subdirs.gyp', chdir='src')

test.relocate('src', 'relocate/src')

# Test that we build all targets.
test.build('subdirs.gyp', 'target', chdir='relocate/src')
test.must_exist('relocate/src/subdir1/action1.txt')
test.must_exist('relocate/src/subdir2/action2.txt')

# Test that we build all targets using the correct actions, even if they have
# the same names.
test.build('subdirs.gyp', 'target_same_action_name', chdir='relocate/src')
test.must_exist('relocate/src/subdir1/action.txt')
test.must_exist('relocate/src/subdir2/action.txt')

# Test that we build all targets using the correct rules, even if they have
# the same names.
test.build('subdirs.gyp', 'target_same_rule_name', chdir='relocate/src')
test.must_exist('relocate/src/subdir1/rule.txt')
test.must_exist('relocate/src/subdir2/rule.txt')

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2009 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies building a target and a subsidiary dependent target from a
.gyp file in a subdirectory, without specifying an explicit output build
directory, and using the generated solution or project file at the top
of the tree as the entry point.
                                 
The configuration sets the Xcode SYMROOT variable and uses --depth=
to make Xcode behave like the other build tools--that is, put all
built targets in a single output build directory at the top of the tree.
"""

import TestGyp

test = TestGyp.TestGyp()

test.run_gyp('prog1.gyp', '-Dset_symroot=1', '--depth=.', chdir='src')

test.relocate('src', 'relocate/src')

# Suppress the test infrastructure's setting SYMROOT on the command line.
test.build('prog1.gyp', test.ALL, SYMROOT=None, chdir='relocate/src')

test.run_built_executable('prog1',
                          stdout="Hello from prog1.c\n",
                          chdir='relocate/src')
test.run_built_executable('prog2',
                          stdout="Hello from prog2.c\n",
                          chdir='relocate/src')

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Make sure warning-as-error is extracted properly.
"""

import TestGyp

import sys

if sys.platform == 'win32':
  test = TestGyp.TestGyp(formats=['msvs', 'ninja'])

  CHDIR = 'compiler-flags'
  test.run_gyp('warning-as-error.gyp', chdir=CHDIR)

  # The source file contains a warning, so if WarnAsError is false (or
  # default, which is also false), then the build should succeed, otherwise it
  # must fail.

  test.build('warning-as-error.gyp', 'test_warn_as_error_false', chdir=CHDIR)
  test.build('warning-as-error.gyp', 'test_warn_as_error_unset', chdir=CHDIR)
  test.build('warning-as-error.gyp', 'test_warn_as_error_true', chdir=CHDIR,
    status=1)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Tests that filenames that contain colons are handled correctly.
(This is important for absolute paths on Windows.)
"""

import os
import sys
import TestGyp

# TODO: Make colons in filenames work with make, if required.
test = TestGyp.TestGyp(formats=['!make'])
CHDIR = 'colon'

source_name = 'colon/a:b.c'
copies_name = 'colon/a:b.c-d'
if sys.platform == 'win32':
  # Windows uses : as drive separator and doesn't allow it in regular filenames.
  # Use abspath() to create a path that contains a colon instead.
  abs_source = os.path.abspath('colon/file.c')
  test.write('colon/test.gyp',
             test.read('colon/test.gyp').replace("'a:b.c'", repr(abs_source)))
  source_name = abs_source

  abs_copies = os.path.abspath('colon/file.txt')
  test.write('colon/test.gyp',
             test.read('colon/test.gyp').replace("'a:b.c-d'", repr(abs_copies)))
  copies_name = abs_copies

# Create the file dynamically, Windows is unhappy if a file with a colon in
# its name is checked in.
test.write(source_name, 'int main() {}')
test.write(copies_name, 'foo')

test.run_gyp('test.gyp', chdir=CHDIR)
test.build('test.gyp', test.ALL, chdir=CHDIR)
test.built_file_must_exist(os.path.basename(copies_name), chdir=CHDIR)
test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Make sure warning level is extracted properly.
"""

import TestGyp

import sys

if sys.platform == 'win32':
  test = TestGyp.TestGyp(formats=['msvs', 'ninja'])

  CHDIR = 'compiler-flags'
  test.run_gyp('warning-level.gyp', chdir=CHDIR)

  # A separate target for each warning level: one pass (compiling a file
  # containing a warning that's above the specified level); and one fail
  # (compiling a file at the specified level). No pass for 4 of course,
  # because it would have to have no warnings. The default warning level is
  # equivalent to level 1.

  test.build('warning-level.gyp', 'test_wl1_fail', chdir=CHDIR, status=1)
  test.build('warning-level.gyp', 'test_wl1_pass', chdir=CHDIR)

  test.build('warning-level.gyp', 'test_wl2_fail', chdir=CHDIR, status=1)
  test.build('warning-level.gyp', 'test_wl2_pass', chdir=CHDIR)

  test.build('warning-level.gyp', 'test_wl3_fail', chdir=CHDIR, status=1)
  test.build('warning-level.gyp', 'test_wl3_pass', chdir=CHDIR)

  test.build('warning-level.gyp', 'test_wl4_fail', chdir=CHDIR, status=1)

  test.build('warning-level.gyp', 'test_def_fail', chdir=CHDIR, status=1)
  test.build('warning-level.gyp', 'test_def_pass', chdir=CHDIR)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2009 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.
import sys

contents = open(sys.argv[1], 'r').read()
open(sys.argv[2], 'wb').write(contents)

sys.exit(0)



#!/usr/bin/env python
# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.
"""
Verifies that make_global_settings can be used to override the
compiler settings.
"""

import TestGyp
import os
import copy
import sys
from string import Template


if sys.platform == 'win32':
  # cross compiling not support by ninja on windows
  # and make not supported on windows at all.
  sys.exit(0)

test = TestGyp.TestGyp(formats=['ninja', 'make'])

gypfile = 'compiler-global-settings.gyp'

replacements = { 'PYTHON': '/usr/bin/python', 'PWD': os.getcwd()}

# Process the .in gyp file to produce the final gyp file
# since we need to include absolute paths in the make_global_settings
# section.
replacements['TOOLSET'] = 'target'
s = Template(open(gypfile + '.in').read())
output = open(gypfile, 'w')
output.write(s.substitute(replacements))
output.close()

test.run_gyp(gypfile)
test.build(gypfile)
test.must_contain_all_lines(test.stdout(), ['my_cc.py', 'my_cxx.py', 'FOO'])

# Same again but with the host toolset.
replacements['TOOLSET'] = 'host'
s = Template(open(gypfile + '.in').read())
output = open(gypfile, 'w')
output.write(s.substitute(replacements))
output.close()

test.run_gyp(gypfile)
test.build(gypfile)
test.must_contain_all_lines(test.stdout(), ['my_cc.py', 'my_cxx.py', 'BAR'])

test.pass_test()



#!/usr/bin/python
# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

import sys

src = open(sys.argv[1])
dst = open(sys.argv[2], 'w')
for ch in src.read():
  dst.write('%d,\n' % ord(ch))
src.close()
dst.close()



#!/usr/bin/env python

# Copyright (c) 2011 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies build of an executable with C++ defines.
"""

import TestGyp

test = TestGyp.TestGyp()

test.run_gyp('defines.gyp')

test.build('defines.gyp')

expect = """\
FOO is defined
VALUE is 1
2*PAREN_VALUE is 12
HASH_VALUE is a#1
"""
test.run_built_executable('defines', stdout=expect)

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that a project hierarchy created with the --generator-output=
option can be built even when it's relocated to a different path.
"""

import TestGyp
import os

test = TestGyp.TestGyp()

test.run_gyp('standalone.gyp', '-Gstandalone')

# Look at all the files in the tree to make sure none
# of them reference the gyp file.
for root, dirs, files in os.walk("."):
  for file in files:
    # ignore ourself
    if os.path.splitext(__file__)[0] in file:
      continue
    file = os.path.join(root, file)
    contents = open(file).read()
    if 'standalone.gyp' in contents:
      print 'gyp file referenced in generated output: %s' % file
      test.fail_test()


test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verify that building an object file correctly depends on running actions in
dependent targets, but not the targets themselves.
"""

import os
import sys
import TestGyp

# NOTE(piman): This test will not work with other generators because:
# - it explicitly tests the optimization, which is not implemented (yet?) on
# other generators
# - it relies on the exact path to output object files, which is generator
# dependent, and actually, relies on the ability to build only that object file,
# which I don't think is available on all generators.
# TODO(piman): Extend to other generators when possible.
test = TestGyp.TestGyp(formats=['ninja'])

test.run_gyp('action_dependencies.gyp', chdir='src')

chdir = 'relocate/src'
test.relocate('src', chdir)

objext = '.obj' if sys.platform == 'win32' else '.o'

test.build('action_dependencies.gyp',
           os.path.join('obj', 'b.b' + objext),
           chdir=chdir)

# The 'a' actions should be run (letting b.c compile), but the a static library
# should not be built.
test.built_file_must_not_exist('a', type=test.STATIC_LIB, chdir=chdir)
test.built_file_must_not_exist('b', type=test.STATIC_LIB, chdir=chdir)
test.built_file_must_exist(os.path.join('obj', 'b.b' + objext), chdir=chdir)

test.build('action_dependencies.gyp',
           os.path.join('obj', 'c.c' + objext),
           chdir=chdir)

# 'a' and 'b' should be built, so that the 'c' action succeeds, letting c.c
# compile
test.built_file_must_exist('a', type=test.STATIC_LIB, chdir=chdir)
test.built_file_must_exist('b', type=test.EXECUTABLE, chdir=chdir)
test.built_file_must_exist(os.path.join('obj', 'c.c' + objext), chdir=chdir)


test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2011 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies rules related variables are expanded.
"""

import TestGyp

test = TestGyp.TestGyp(formats=['ninja'])

test.relocate('src', 'relocate/src')

test.run_gyp('variables.gyp', chdir='relocate/src')

test.build('variables.gyp', chdir='relocate/src')

test.run_built_executable('all_rule_variables',
                          chdir='relocate/src',
                          stdout="input_root\ninput_dirname\ninput_path\n" +
                          "input_ext\ninput_name\n")

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2009 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that a default gyp define can be overridden.
"""

import os
import TestGyp

test = TestGyp.TestGyp()

# Command-line define
test.run_gyp('defines.gyp', '-D', 'OS=fakeos')
test.build('defines.gyp')
test.built_file_must_exist('fakeosprogram', type=test.EXECUTABLE)
# Clean up the exe so subsequent tests don't find an old exe.
os.remove(test.built_file_path('fakeosprogram', type=test.EXECUTABLE))

# Without "OS" override, fokeosprogram shouldn't be built.
test.run_gyp('defines.gyp')
test.build('defines.gyp')
test.built_file_must_not_exist('fakeosprogram', type=test.EXECUTABLE)

# Environment define
os.environ['GYP_DEFINES'] = 'OS=fakeos'
test.run_gyp('defines.gyp')
test.build('defines.gyp')
test.built_file_must_exist('fakeosprogram', type=test.EXECUTABLE)

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies actions can be in 'none' type targets with source files.
"""

import TestGyp

test = TestGyp.TestGyp()
# TODO(bradnelson): fix scons.
if test.format == 'scons':
  test.skip_test()

test.run_gyp('none_with_source_files.gyp', chdir='src')
test.relocate('src', 'relocate/src')
test.build('none_with_source_files.gyp', chdir='relocate/src')

file_content = 'foo.cc\n'

test.built_file_must_match('fake.out', file_content, chdir='relocate/src')

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that 'copies' with app bundles are handled correctly.
"""

import TestGyp

import os
import sys
import time

if sys.platform == 'darwin':
  test = TestGyp.TestGyp(formats=['ninja', 'make', 'xcode'])

  test.run_gyp('framework.gyp', chdir='framework')

  test.build('framework.gyp', 'copy_target', chdir='framework')

  # Check that the copy succeeded.
  test.built_file_must_exist(
      'Test Framework.framework/foo/Dependency Bundle.framework',
      chdir='framework')
  test.built_file_must_exist(
      'Test Framework.framework/foo/Dependency Bundle.framework/Versions/A',
      chdir='framework')
  test.built_file_must_exist(
      'Test Framework.framework/Versions/A/Libraries/empty.c',
      chdir='framework')


  # Check that rebuilding the target a few times works.
  dep_bundle = test.built_file_path('Dependency Bundle.framework',
                                    chdir='framework')
  mtime = os.path.getmtime(dep_bundle)
  atime = os.path.getatime(dep_bundle)
  for i in range(3):
    os.utime(dep_bundle, (atime + i * 1000, mtime + i * 1000))
    test.build('framework.gyp', 'copy_target', chdir='framework')


  # Check that actions ran.
  test.built_file_must_exist('action_file', chdir='framework')

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Make sure .s files aren't passed to cl.
"""

import TestGyp

import sys

if sys.platform == 'win32':
  test = TestGyp.TestGyp(formats=['msvs', 'ninja'])

  CHDIR = 'asm-files'
  test.run_gyp('asm-files.gyp', chdir=CHDIR)
  # The compiler will error out if it's passed the .s files, so just make sure
  # the build succeeds. The compiler doesn't directly support building
  # assembler files on Windows, they have to be built explicitly with a
  # third-party tool.
  test.build('asm-files.gyp', test.ALL, chdir=CHDIR)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verify that dependent rules are executed iff a dependency action modifies its
outputs.
"""

import TestGyp
import os

test = TestGyp.TestGyp(formats=['ninja', 'make', 'xcode'])

test.run_gyp('restat.gyp', chdir='src')

chdir = 'relocate/src'
test.relocate('src', chdir)

# Building 'dependent' the first time generates 'side_effect', but building it
# the second time doesn't, because 'create_intermediate' doesn't update its
# output.
test.build('restat.gyp', 'dependent', chdir=chdir)
test.built_file_must_exist('side_effect', chdir=chdir)
os.remove(test.built_file_path('side_effect', chdir=chdir))
test.build('restat.gyp', 'dependent', chdir=chdir)
test.built_file_must_not_exist('side_effect', chdir=chdir)

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that a dependency on a bundle causes the whole bundle to be built.
"""

import TestGyp

import sys

if sys.platform == 'darwin':
  test = TestGyp.TestGyp(formats=['ninja', 'make', 'xcode'])

  test.run_gyp('test.gyp', chdir='depend-on-bundle')

  test.build('test.gyp', 'dependent_on_bundle', chdir='depend-on-bundle')

  # Binary itself.
  test.built_file_must_exist('dependent_on_bundle', chdir='depend-on-bundle')

  # Bundle dependency.
  test.built_file_must_exist(
      'my_bundle.framework/Versions/A/my_bundle',
      chdir='depend-on-bundle')
  test.built_file_must_exist(  # package_framework
      'my_bundle.framework/my_bundle',
      chdir='depend-on-bundle')
  test.built_file_must_exist(  # plist
      'my_bundle.framework/Versions/A/Resources/Info.plist',
      chdir='depend-on-bundle')
  test.built_file_must_exist(
      'my_bundle.framework/Versions/A/Resources/English.lproj/'  # Resources
      'InfoPlist.strings',
      chdir='depend-on-bundle')

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Make sure debug format settings are extracted properly.
"""

import TestGyp

import sys

if sys.platform == 'win32':
  test = TestGyp.TestGyp(formats=['ninja'])

  CHDIR = 'compiler-flags'
  test.run_gyp('debug-format.gyp', chdir=CHDIR)

  # While there's ways to via .pdb contents, the .pdb doesn't include
  # which style the debug information was created from, so we resort to just
  # verifying the flags are correct on the command line.

  ninja_file = test.built_file_path('obj/test-debug-format-off.ninja',
      chdir=CHDIR)
  test.must_not_contain(ninja_file, '/Z7')
  test.must_not_contain(ninja_file, '/Zi')
  test.must_not_contain(ninja_file, '/ZI')

  ninja_file = test.built_file_path('obj/test-debug-format-oldstyle.ninja',
      chdir=CHDIR)
  test.must_contain(ninja_file, '/Z7')

  ninja_file = test.built_file_path('obj/test-debug-format-pdb.ninja',
      chdir=CHDIR)
  test.must_contain(ninja_file, '/Zi')

  ninja_file = test.built_file_path('obj/test-debug-format-editcontinue.ninja',
      chdir=CHDIR)
  test.must_contain(ninja_file, '/ZI')

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Make sure pdb is named as expected (shared between .cc files).
"""

import TestGyp

import sys

if sys.platform == 'win32':
  test = TestGyp.TestGyp(formats=['ninja'])

  CHDIR = 'compiler-flags'
  test.run_gyp('pdbname.gyp', chdir=CHDIR)
  test.build('pdbname.gyp', test.ALL, chdir=CHDIR)

  # Confirm that the default behaviour is to name the .pdb per-target (rather
  # than per .cc file).
  test.built_file_must_exist('test_pdbname.pdb', chdir=CHDIR)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2009 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verify that a hard_dependency that is exported is pulled in as a dependency
for a target if the target is a static library and if the generator will
remove dependencies between static libraries.
"""

import TestGyp

test = TestGyp.TestGyp()

if test.format == 'dump_dependency_json':
  test.skip_test('Skipping test; dependency JSON does not adjust ' \
                 'static libraries.\n')

test.run_gyp('hard_dependency.gyp', chdir='src')

chdir = 'relocate/src'
test.relocate('src', chdir)

test.build('hard_dependency.gyp', 'c', chdir=chdir)

# The 'a' static library should be built, as it has actions with side-effects
# that are necessary to compile 'c'. Even though 'c' does not directly depend
# on 'a', because 'a' is a hard_dependency that 'b' exports, 'c' should import
# it as a hard_dependency and ensure it is built before building 'c'.
test.built_file_must_exist('a', type=test.STATIC_LIB, chdir=chdir)
test.built_file_must_not_exist('b', type=test.STATIC_LIB, chdir=chdir)
test.built_file_must_exist('c', type=test.STATIC_LIB, chdir=chdir)
test.built_file_must_not_exist('d', type=test.STATIC_LIB, chdir=chdir)

test.pass_test()



#!/usr/bin/env python
# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.
import sys
print sys.argv



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that missing 'sources' files are treated as fatal errors when the
the generator flag 'msvs_error_on_missing_sources' is set.
"""

import TestGyp
import os
import sys

if sys.platform == 'win32':
  test = TestGyp.TestGyp(formats=['msvs', 'ninja'], workdir='workarea_all')

  # With the flag not set
  test.run_gyp('hello_missing.gyp')

  # With the flag explicitly set to 0
  try:
    os.environ['GYP_GENERATOR_FLAGS'] = 'msvs_error_on_missing_sources=0'
    test.run_gyp('hello_missing.gyp')
  finally:
    del os.environ['GYP_GENERATOR_FLAGS']

  # With the flag explicitly set to 1
  try:
    os.environ['GYP_GENERATOR_FLAGS'] = 'msvs_error_on_missing_sources=1'
    # Test to make sure GYP raises an exception (exit status 1). Since this will
    # also print a backtrace, ensure that TestGyp is not checking that stderr is
    # empty by specifying None, which means do not perform any checking.
    # Instead, stderr is checked below to ensure it contains the expected
    # output.
    test.run_gyp('hello_missing.gyp', status=1, stderr=None)
  finally:
    del os.environ['GYP_GENERATOR_FLAGS']
  test.must_contain_any_line(test.stderr(),
                            ["Missing input files:"])

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Make sure additional manual compiler flags are extracted properly.
"""

import TestGyp

import sys

if sys.platform == 'win32':
  test = TestGyp.TestGyp(formats=['msvs', 'ninja'])

  CHDIR = 'compiler-flags'
  test.run_gyp('additional-options.gyp', chdir=CHDIR)

  # Warning level not overidden, must fail.
  test.build('additional-options.gyp', 'test_additional_none', chdir=CHDIR,
      status=1)

  # Warning level is overridden, must succeed.
  test.build('additional-options.gyp', 'test_additional_one', chdir=CHDIR)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies mac bundles work with --generator-output.
"""

import TestGyp

import sys

if sys.platform == 'darwin':
  # Ninja doesn't support --generator-output.
  test = TestGyp.TestGyp(formats=['!ninja'])

  MAC_BUNDLE_DIR = 'mac-bundle'
  GYPFILES_DIR = 'gypfiles'
  test.writable(test.workpath(MAC_BUNDLE_DIR), False)
  test.run_gyp('test.gyp',
               '--generator-output=' + test.workpath(GYPFILES_DIR),
               chdir=MAC_BUNDLE_DIR)
  test.writable(test.workpath(MAC_BUNDLE_DIR), True)

  test.build('test.gyp', test.ALL, chdir=GYPFILES_DIR)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that OTHER_LDFLAGS is passed to libtool.
"""

import TestGyp

import sys

if sys.platform == 'darwin':
  test = TestGyp.TestGyp(formats=['ninja', 'make', 'xcode'],
                         match = lambda a, b: True)

  build_error_code = {
    'xcode': 1,
    'make': 2,
    'ninja': 1,
  }[test.format]

  CHDIR = 'ldflags-libtool'
  test.run_gyp('test.gyp', chdir=CHDIR)

  test.build('test.gyp', 'ldflags_passed_to_libtool', chdir=CHDIR,
             status=build_error_code)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Handle macro expansion in inputs and outputs of rules.
"""

import TestGyp

import sys

if sys.platform == 'win32':
  test = TestGyp.TestGyp(formats=['msvs', 'ninja'])

  CHDIR = 'vs-macros'
  test.run_gyp('input-output-macros.gyp', chdir=CHDIR)

  test.build('input-output-macros.gyp', 'test_expansions', chdir=CHDIR)

  test.built_file_must_exist('stuff.blah.something',
      content='Random data file.\nModified.',
      chdir=CHDIR)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that ^(latelate) style variables work.
"""

import TestGyp

test = TestGyp.TestGyp()

test.run_gyp('latelate.gyp', chdir='src')

test.relocate('src', 'relocate/src')

test.build('latelate.gyp', test.ALL, chdir='relocate/src')

test.run_built_executable(
    'program', chdir='relocate/src', stdout='program.cc\n')


test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2010 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Check that duplicate targets in a directory gives an error.
"""

import TestGyp

test = TestGyp.TestGyp()

# Require that gyp files with duplicate targets spit out an error.
test.run_gyp('all.gyp', chdir='src', status=1, stderr=None)

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Tests the use of rules with the same output file name.
"""

import TestGyp

test = TestGyp.TestGyp()

test.run_gyp('subdirs.gyp', chdir='src')

test.relocate('src', 'relocate/src')

test.build('subdirs.gyp', test.ALL, chdir='relocate/src')
test.must_exist('relocate/src/subdir1/rule.txt')
test.must_exist('relocate/src/subdir2/rule.txt')

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that msvs_list_excluded_files=0 doesn't list files that would
normally be in _excluded_files, and that if that flag is not set, then they
are still listed.
"""

import os
import TestGyp

test = TestGyp.TestGyp(formats=['msvs'], workdir='workarea_all')


# with the flag set to 0
try:
  os.environ['GYP_GENERATOR_FLAGS'] = 'msvs_list_excluded_files=0'
  test.run_gyp('hello_exclude.gyp')
finally:
  del os.environ['GYP_GENERATOR_FLAGS']
if test.uses_msbuild:
  test.must_not_contain('hello.vcxproj', 'hello_mac')
else:
  test.must_not_contain('hello.vcproj', 'hello_mac')


# with the flag not set
test.run_gyp('hello_exclude.gyp')
if test.uses_msbuild:
  test.must_contain('hello.vcxproj', 'hello_mac')
else:
  test.must_contain('hello.vcproj', 'hello_mac')


# with the flag explicitly set to 1
try:
  os.environ['GYP_GENERATOR_FLAGS'] = 'msvs_list_excluded_files=1'
  test.run_gyp('hello_exclude.gyp')
finally:
  del os.environ['GYP_GENERATOR_FLAGS']
if test.uses_msbuild:
  test.must_contain('hello.vcxproj', 'hello_mac')
else:
  test.must_contain('hello.vcproj', 'hello_mac')


test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2009 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that toolsets are correctly applied
"""

import TestGyp

# Multiple toolsets are currently only supported by the make generator.
test = TestGyp.TestGyp(formats=['make'])

test.run_gyp('toolsets.gyp')

test.build('toolsets.gyp', test.ALL)

test.run_built_executable('host-main', stdout="Host\n")
test.run_built_executable('target-main', stdout="Target\n")

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2009 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies simplest-possible build of a "Hello, world!" program
using an explicit build target of 'hello'.
"""

import TestGyp

test = TestGyp.TestGyp(workdir='workarea_target')

test.run_gyp('hello.gyp')

test.build('hello.gyp', 'hello')

test.run_built_executable('hello', stdout="Hello, world!\n")

test.up_to_date('hello.gyp', 'hello')

test.pass_test()



#!/usr/bin/python
# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.


import sys

fh = open(sys.argv[-1], 'wb')
for filename in sys.argv[1:-1]:
  fh.write(open(filename).read())
fh.close()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that dependent Xcode settings are processed correctly.
"""

import TestGyp

import sys

if sys.platform == 'darwin':
  test = TestGyp.TestGyp(formats=['ninja', 'make', 'xcode'])

  CHDIR = 'xcode-env-order'
  INFO_PLIST_PATH = 'Test.app/Contents/Info.plist'

  test.run_gyp('test.gyp', chdir=CHDIR)
  test.build('test.gyp', test.ALL, chdir=CHDIR)

  # Env vars in 'copies' filenames.
  test.built_file_must_exist('Test-copy-brace/main.c', chdir=CHDIR)
  test.built_file_must_exist('Test-copy-paren/main.c', chdir=CHDIR)
  test.built_file_must_exist('Test-copy-bare/main.c', chdir=CHDIR)

  # Env vars in 'actions' filenames and inline actions
  test.built_file_must_exist('action-copy-brace.txt', chdir=CHDIR)
  test.built_file_must_exist('action-copy-paren.txt', chdir=CHDIR)
  test.built_file_must_exist('action-copy-bare.txt', chdir=CHDIR)

  # Env vars in 'rules' filenames and inline actions
  test.built_file_must_exist('rule-copy-brace.txt', chdir=CHDIR)
  test.built_file_must_exist('rule-copy-paren.txt', chdir=CHDIR)
  # TODO: see comment in test.gyp for this file.
  #test.built_file_must_exist('rule-copy-bare.txt', chdir=CHDIR)

  # Env vars in Info.plist.
  info_plist = test.built_file_path(INFO_PLIST_PATH, chdir=CHDIR)
  test.must_exist(info_plist)

  test.must_contain(info_plist, '''\
\t<key>BraceProcessedKey1</key>
\t<string>D:/Source/Project/Test</string>''')
  test.must_contain(info_plist, '''\
\t<key>BraceProcessedKey2</key>
\t<string>/Source/Project/Test</string>''')
  test.must_contain(info_plist, '''\
\t<key>BraceProcessedKey3</key>
\t<string>com.apple.product-type.application:D:/Source/Project/Test</string>''')

  test.must_contain(info_plist, '''\
\t<key>ParenProcessedKey1</key>
\t<string>D:/Source/Project/Test</string>''')
  test.must_contain(info_plist, '''\
\t<key>ParenProcessedKey2</key>
\t<string>/Source/Project/Test</string>''')
  test.must_contain(info_plist, '''\
\t<key>ParenProcessedKey3</key>
\t<string>com.apple.product-type.application:D:/Source/Project/Test</string>''')

  test.must_contain(info_plist, '''\
\t<key>BareProcessedKey1</key>
\t<string>D:/Source/Project/Test</string>''')
  test.must_contain(info_plist, '''\
\t<key>BareProcessedKey2</key>
\t<string>/Source/Project/Test</string>''')
  # NOTE: For bare variables, $PRODUCT_TYPE is not replaced! It _is_ replaced
  # if it's not right at the start of the string (e.g. ':$PRODUCT_TYPE'), so
  # this looks like an Xcode bug. This bug isn't emulated (yet?), so check this
  # only for Xcode.
  if test.format == 'xcode':
    test.must_contain(info_plist, '''\
\t<key>BareProcessedKey3</key>
\t<string>$PRODUCT_TYPE:D:/Source/Project/Test</string>''')

  test.must_contain(info_plist, '''\
\t<key>MixedProcessedKey</key>
\t<string>/Source/Project:Test:mh_execute</string>''')

  test.pass_test()



# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

import os
import sys

if not os.path.exists(sys.argv[1]):
  raise
open(sys.argv[2], 'w').close()



#!/usr/bin/env python

# Copyright (c) 2011 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that postbuild steps work.
"""

import TestGyp

import sys

if sys.platform == 'darwin':
  test = TestGyp.TestGyp(formats=['ninja', 'make', 'xcode'])

  test.run_gyp('test.gyp', chdir='postbuilds')

  test.build('test.gyp', test.ALL, chdir='postbuilds')

  # See comment in test/subdirectory/gyptest-subdir-default.py
  if test.format == 'xcode':
    chdir = 'postbuilds/subdirectory'
  else:
    chdir = 'postbuilds'

  # Created by the postbuild scripts
  test.built_file_must_exist('el.a_touch',
                             type=test.STATIC_LIB,
                             chdir='postbuilds')
  test.built_file_must_exist('el.a_gyp_touch',
                             type=test.STATIC_LIB,
                             chdir='postbuilds')
  test.built_file_must_exist('nest_el.a_touch',
                             type=test.STATIC_LIB,
                             chdir=chdir)
  test.built_file_must_exist(
      'dyna.framework/Versions/A/dyna_touch',
      chdir='postbuilds')
  test.built_file_must_exist(
      'dyna.framework/Versions/A/dyna_gyp_touch',
      chdir='postbuilds')
  test.built_file_must_exist(
      'nest_dyna.framework/Versions/A/nest_dyna_touch',
      chdir=chdir)
  test.built_file_must_exist('dyna_standalone.dylib_gyp_touch',
                             type=test.SHARED_LIB,
                             chdir='postbuilds')
  test.built_file_must_exist('copied_file.txt', chdir='postbuilds')
  test.built_file_must_exist('copied_file_2.txt', chdir=chdir)

  test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verifies that a rule that generates multiple outputs rebuilds
correctly when the inputs change.
"""

import TestGyp

test = TestGyp.TestGyp(workdir='workarea_default')

test.run_gyp('same_target.gyp', chdir='src')

test.relocate('src', 'relocate/src')


test.build('same_target.gyp', chdir='relocate/src')

expect = """\
Hello from main.c
Hello from prog1.in!
Hello from prog2.in!
"""

test.run_built_executable('program', chdir='relocate/src', stdout=expect)

test.up_to_date('same_target.gyp', 'program', chdir='relocate/src')


test.sleep()
contents = test.read(['relocate', 'src', 'prog1.in'])
contents = contents.replace('!', ' AGAIN!')
test.write(['relocate', 'src', 'prog1.in'], contents)

test.build('same_target.gyp', chdir='relocate/src')

expect = """\
Hello from main.c
Hello from prog1.in AGAIN!
Hello from prog2.in!
"""

test.run_built_executable('program', chdir='relocate/src', stdout=expect)

test.up_to_date('same_target.gyp', 'program', chdir='relocate/src')


test.sleep()
contents = test.read(['relocate', 'src', 'prog2.in'])
contents = contents.replace('!', ' AGAIN!')
test.write(['relocate', 'src', 'prog2.in'], contents)

test.build('same_target.gyp', chdir='relocate/src')

expect = """\
Hello from main.c
Hello from prog1.in AGAIN!
Hello from prog2.in AGAIN!
"""

test.run_built_executable('program', chdir='relocate/src', stdout=expect)

test.up_to_date('same_target.gyp', 'program', chdir='relocate/src')


# Test that modifying a rule's inputs (specifically, make-sources.py) causes
# the targets to be built.

test.sleep()
contents = test.read(['relocate', 'src', 'make-sources.py'])
contents = contents.replace('%s', 'the amazing %s')
test.write(['relocate', 'src', 'make-sources.py'], contents)

test.build('same_target.gyp', chdir='relocate/src')

expect = """\
Hello from main.c
Hello from the amazing prog1.in AGAIN!
Hello from the amazing prog2.in AGAIN!
"""

test.run_built_executable('program', chdir='relocate/src', stdout=expect)

test.up_to_date('same_target.gyp', 'program', chdir='relocate/src')


test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Build a .gyp that depends on 2 gyp files with the same name.
"""

import TestGyp

test = TestGyp.TestGyp()

test.run_gyp('all.gyp', chdir='src')

test.relocate('src', 'relocate/src')

test.build('all.gyp', test.ALL, chdir='relocate/src')

expect1 = """\
Hello from main1.cc
"""

expect2 = """\
Hello from main2.cc
"""

if test.format == 'xcode':
  chdir1 = 'relocate/src/subdir1'
  chdir2 = 'relocate/src/subdir2'
else:
  chdir1 = chdir2 = 'relocate/src'

test.run_built_executable('program1', chdir=chdir1, stdout=expect1)
test.run_built_executable('program2', chdir=chdir2, stdout=expect2)

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2012 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Test that two targets with the same name generates an error.
"""

import TestGyp
import TestCmd

# TODO(sbc): Remove the need for match_re below, and make scons
# error messages consistent with other generators by removing
# input.py:generator_wants_absolute_build_file_paths.

test = TestGyp.TestGyp()

stderr = ('gyp: Duplicate target definitions for '
          '.*duplicate_targets.gyp:foo#target\n')
test.run_gyp('duplicate_targets.gyp', status=1, stderr=stderr,
             match=TestCmd.match_re)

stderr = ('gyp: Unable to find targets in build file .*missing_targets.gyp '
          'while trying to load missing_targets.gyp\n')
test.run_gyp('missing_targets.gyp', status=1, stderr=stderr,
             match=TestCmd.match_re)

stderr = ('gyp: rule bar exists in duplicate, target '
          '.*duplicate_rule.gyp:foo#target\n')
test.run_gyp('duplicate_rule.gyp', status=1, stderr=stderr,
             match=TestCmd.match_re)

stderr = ("gyp: Key 'targets' repeated at level 1 with key path '' while "
          "reading .*duplicate_node.gyp while trying to load "
          "duplicate_node.gyp\n")
test.run_gyp('duplicate_node.gyp', '--check', status=1, stderr=stderr,
             match=TestCmd.match_re)

stderr = 'gyp: Duplicate basenames in sources section, see list above\n'
test.run_gyp('duplicate_basenames.gyp', status=1, stderr=stderr)

stderr = ("gyp: Dependency '.*missing_dep.gyp:missing.gyp#target' not found "
          "while trying to load target .*missing_dep.gyp:foo#target\n")
test.run_gyp('missing_dep.gyp', status=1, stderr=stderr,
             match=TestCmd.match_re)

test.pass_test()



#!/usr/bin/env python

# Copyright (c) 2011 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Verify that a link time only dependency will get pulled into the set of built
targets, even if no executable uses it.
"""

import TestGyp

import sys

test = TestGyp.TestGyp()

test.run_gyp('lib_only.gyp')

test.build('lib_only.gyp', test.ALL)

test.built_file_must_exist('a', type=test.STATIC_LIB)

# TODO(bradnelson/mark):
# On linux and windows a library target will at least pull its link dependencies
# into the generated sln/_main.scons, since not doing so confuses users.
# This is not currently implemented on mac, which has the opposite behavior.
if sys.platform == 'darwin':
  if test.format == 'xcode':
    test.built_file_must_not_exist('b', type=test.STATIC_LIB)
  else:
    assert test.format in ('make', 'ninja')
    test.built_file_must_exist('b', type=test.STATIC_LIB)
else:
  # Make puts the resulting library in a directory matching the input gyp file;
  # for the 'b' library, that is in the 'b' subdirectory.
  test.built_file_must_exist('b', type=test.STATIC_LIB, subdir='b')

test.pass_test()
